#version 120
#define USE_GPU4    1
#define USE_NORMAL_MAP 1
#define SHADOW_QUALITY  2

#define PER_FRAGMENT_SPECULAR   1
#define PER_FRAGMENT_LIGHT_CALCS  (0 || PER_FRAGMENT_SPECULAR)

#define MANUAL_SHADOW_PROJECTION    1

#if USE_GPU4
#extension GL_EXT_gpu_shader4 : enable
#endif

varying vec3 vN;
varying vec2 texCoord;
varying vec2 weightCoord;
varying vec4 shadowTexCoord;

varying float dist_from_eye;

uniform sampler2D colorMap;
uniform sampler2D normalMap;
uniform sampler2DShadow shadowMap;
//uniform sampler2D specularMap;

uniform int hasNormalMap;
//uniform int hasSpecularMap;
uniform float ersatzRadiosity;

#if PER_FRAGMENT_LIGHT_CALCS
varying vec3 L0, E0, R0, L1, R1;
varying vec3 L[6];
#else
varying vec3 L0, L1;
varying vec3 L[6];
#endif

//uniform float textureWidth;
//uniform float textureHeight;

uniform int currentlyShadowed;

//varying vec4 vtx_base;

#if !PER_FRAGMENT_SPECULAR
varying float spec_term0, spec_term1;
#endif

//varying float fogFactor;
uniform int shouldUseFog;


uniform int numTextureUnits;

void main (void)
{
    //discard;
    vec4 base;
    
    {
        base = texture2D(colorMap, texCoord); //, 0.0);
    }

    // if this is on, it means parallax is OFF
    vec2 newCoords = texCoord;
    //vec4 shadowTexCoord = gl_TexCoord[1];

    const float two = 2.0;
    const float one = 1.0;
    float multiplier = 2.14; //1.618;
    
#if USE_NORMAL_MAP
    vec4 bump = (hasNormalMap == 1 ? -(two /*1415*/ * texture2D(normalMap, newCoords) - one) : vec4(vN, one));
#else
    vec4 bump = vec4(vN, one);
#endif

//if (hasNormalMap == 0)
//    base = mix(base, vec4(1.0, 0.0, 0.0, 1.0), 0.5);

    if (
#if USE_NORMAL_MAP
    hasNormalMap == 0 &&
#endif
    dist_from_eye < 100.0) // && currentlyShadowed == 0)
    {
	// default gray
	const vec3 grayscale = vec3(0.299, 0.587, 0.114);

        multiplier = 1.0;
        float s11 = dot(base.rgb, grayscale); // * multiplier; //base.r; // dot(base.rgb, grayscale).x; //base.x;
        
        bump.y = s11;

        bump = (two * bump - one);
        bump = normalize(bump);       
        
        //base = mix(base, vec4(1.0, 0.0, 0.0, 1.0), 0.5);
    }
    else if (dist_from_eye >= 100.0)
    {
        multiplier = 0.1;
    }

    ///////////////////
    vec3 N = vN;
    
    N.y += (bump.y * multiplier);
    
    N = normalize(N);
    
    vec4 Iamb = gl_LightModel.ambient * gl_FrontMaterial.ambient * 1.1; //gl_FrontLightProduct[i].ambient; 
    vec4 finalColor = Iamb; //vec4(0.0, 0.0, 0.0, 0.0);
    
    float shiny = gl_FrontMaterial.shininess + 0.1;
    
    // light 0
    //**********
    //calculate Diffuse Term: 
    float diff_term0 = dot(N, L0); //max(dot(N,L0), 0.0);
  
    float shadowValue0 = 1.0;
    
    vec4 newShadowTexCoord;
    
    if (currentlyShadowed == 1 && gl_LightSource[0].position.y > 0.0
        && diff_term0 >= 0.0) // && diff_term0 > 0.0)
    {
#if USE_GPU4

        shadowValue0 = shadow2DProj(shadowMap, shadowTexCoord).x;
        
        
        // outer kernel
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, -1)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, 0)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, 1)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, 2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, 2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(0, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(0, 2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, 2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, -2)).x;
         shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, -1)).x;
         shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, 0)).x;
         shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, 1)).x;
         shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, 2)).x;
        
        // inner kernel
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, -1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, 0)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, 1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(0, -1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(0, 1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, -1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, 0)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, 1)).x * 2.0;

        shadowValue0 /= 44.0;
#else
        shadowValue0 = shadow2DProj(shadowMap, shadowTexCoord).x;
#endif
    }
    
    if (currentlyShadowed == 1 && diff_term0 < 0.0)
    {
        //base = mix(base, vec4(1.0, 0.0, 0.0, 1.0), 0.5);
        shadowValue0 = Iamb.x;
    }
    
    diff_term0 = max(diff_term0, 0.0);
      
    vec4 Idiff0 = gl_FrontLightProduct[0].diffuse * diff_term0;

    Idiff0 = clamp(Idiff0, 0.0, 1.0); 

    // calculate Specular Term:
    
    float point_three = 0.3;
    
    float specularValue = point_three * shiny; //(hasSpecularMap == 1 ? (texture2D(specularMap, newCoords)).x : point_three*shiny);
    vec4 Ispec0;
    Ispec0 = gl_FrontLightProduct[0].specular
        * pow(
#if PER_FRAGMENT_SPECULAR
        max(dot(R0,E0),0.0)
#else
        spec_term0
#endif
        ,specularValue); //point_three*shiny);


    Ispec0 = clamp(Ispec0, 0.0, 1.0); 

    finalColor += (Idiff0 + Ispec0) * shadowValue0;
    //**********

    // light 1
    //**********
    //calculate Diffuse Term: 
    diff_term0 = max(dot(N,L1), 0.0);
  
    Idiff0 = gl_FrontLightProduct[1].diffuse * diff_term0;

    Idiff0 = clamp(Idiff0, 0.0, 1.0); 

    // calculate Specular Term:
    
    Ispec0 = gl_FrontLightProduct[1].specular 
        * pow(
#if PER_FRAGMENT_SPECULAR
        max(dot(R1,E0),0.0)
#else
        spec_term1
#endif
        ,specularValue);


    Ispec0 = clamp(Ispec0, 0.0, 1.0); 

        	vec4 extra_lights;

	for (int i = 0; i < 6; ++i)
	{
		float diff_term = dot(N, L[i]);
		extra_lights += clamp(gl_FrontLightProduct[i].diffuse * diff_term, 0.0, 1.0);
	}
	extra_lights *= Iamb;


    finalColor += (Idiff0 + Ispec0 + extra_lights
		) * shadowValue0;

    //**********

    vec4 final = base * (finalColor);
    
if (shouldUseFog == 1)
{    
const float LOG2 = 1.442695;
    float z = gl_FragCoord.z; // / gl_FragCoord.w;
float fogFactor = exp( -gl_Fog.density *
				   gl_Fog.density * 
				   z * 
				   z * 
				   LOG2 );
fogFactor = clamp(fogFactor, 0.0, 1.0);
final = mix(gl_Fog.color, final, fogFactor);
}
    final.a = base.a * gl_FrontMaterial.diffuse.a * gl_FrontMaterial.specular.a;

    gl_FragColor = final; //texture2D(colorMap, texCoord) * (finalColor);
}