#version 120
#define USE_GPU4    __USE_GPU4__
#define USE_NORMAL_MAP 1
#define SHADOW_QUALITY  __SHADOW_QUALITY__

#define PER_FRAGMENT_LIGHT_CALCS  __PER_FRAGMENT_LIGHT_CALCS__

#define MANUAL_SHADOW_PROJECTION    1

#if USE_GPU4
#extension GL_EXT_gpu_shader4 : enable
#endif

varying vec3 vN;
varying vec2 texCoord;
varying vec2 weightCoord;
varying vec4 shadowTexCoord;

varying float dist_from_eye;

uniform sampler2D colorMap;
uniform sampler2D normalMap;
uniform sampler2DShadow shadowMap;

uniform int hasNormalMap;

varying vec3 L[__NUM_LIGHTS__];

#if PER_FRAGMENT_LIGHT_CALCS
varying float spec_terms[__NUM_LIGHTS__];
varying float A[__NUM_LIGHTS__];
#endif

uniform int numActiveLights;

uniform int currentlyShadowed;

uniform int shouldUseFog;

uniform int numTextureUnits;

void main (void)
{
    //********** setup
    vec4 base = texture2D(colorMap, texCoord);

#if PER_FRAGMENT_LIGHT_CALCS
    // if this is on, it means parallax is OFF
    vec2 newCoords = texCoord;

    const float two = 2.0;
    const float one = 1.0;
    float multiplier = 3.14;
    
#if USE_NORMAL_MAP
    vec4 bump = (hasNormalMap == 1 ? -(two * texture2D(normalMap, newCoords) - one) : vec4(vN, one));
#else
    vec4 bump = vec4(vN, one);
#endif

    if (
#if USE_NORMAL_MAP
    hasNormalMap == 0 &&
#endif
    dist_from_eye < 100.0)
    {
	// default gray
	const vec3 grayscale = vec3(0.299, 0.587, 0.114);

        multiplier = 1.01;
        float s11 = dot(base.rgb, grayscale);
        
        bump.y = s11;

        bump = (two * bump - one);
        bump = normalize(bump);
    }
    else if (dist_from_eye >= 100.0)
    {
        multiplier = 0.1;
    }
    
#endif

    //********** lighting calcs
    vec3 N = vN;
    
#if PER_FRAGMENT_LIGHT_CALCS
    N.y += (bump.y * multiplier);
#endif
    
    N = normalize(N);
    
    vec4 Iamb = gl_LightModel.ambient * gl_FrontMaterial.ambient * 1.1;
    vec4 finalColor = Iamb;
    
    float shadowValue0 = 1.0;
    
    
    float point_three = 0.3;

    if (currentlyShadowed == 1 && gl_LightSource[0].position.y > 0.0
        && dot(N,L[0]) >= 0.0)
    {
#if USE_GPU4
        
        shadowValue0 = shadow2DProj(shadowMap, shadowTexCoord).x;
        
#if SHADOW_QUALITY == 2
        // outer kernel
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, -1)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, 0)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, 1)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-2, 2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, 2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(0, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(0, 2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, 2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, -2)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, -1)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, 0)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, 1)).x;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(2, 2)).x;
#endif
      
#if SHADOW_QUALITY > 0
        // inner kernel
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, -1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, 0)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(-1, 1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(0, -1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(0, 1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, -1)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, 0)).x * 2.0;
        shadowValue0 += shadow2DProjOffset(shadowMap, shadowTexCoord, ivec2(1, 1)).x * 2.0;
#endif
        
#if SHADOW_QUALITY == 2
        shadowValue0 /= 44.0;
#elif SHADOW_QUALITY > 0
        shadowValue0 /= 28.0;
#endif
        
#else
        shadowValue0 = shadow2DProj(shadowMap, shadowTexCoord).x;
#endif
    }
    
    if (currentlyShadowed == 1 && dot(N,L[0]) < 0.0)
        shadowValue0 = Iamb.x;
    
    vec4 lights[__NUM_LIGHTS__];
    
    float diff_term;
    int i = 0;

    for (int i = 0; i < __NUM_LIGHTS__; ++i)
	{
#if PER_FRAGMENT_LIGHT_CALCS
        lights[i] = (clamp(gl_FrontLightProduct[i].diffuse * max(dot(N, L[i]), 0.0), 0.0, 1.0) + clamp(gl_FrontLightProduct[i].specular * spec_terms[i], 0.0, 1.0));
        
        lights[i] = (lights[i] * A[i] * shadowValue0);
#else
        lights[i] = vec4(L[i], 1.0) * shadowValue0;
#endif
        
        finalColor += lights[i];
	}
    
    //**********

    vec4 final = base * (finalColor);
    
if (shouldUseFog == 1)
{    
const float LOG2 = 1.442695;
    float z = gl_FragCoord.z / gl_FragCoord.w;
    float fogFactor = exp2( -gl_Fog.density * 
				   gl_Fog.density * 
				   z * 
				   z * 
				   LOG2 );
fogFactor = clamp(fogFactor, 0.0, 1.0);
final = mix(gl_Fog.color, final, fogFactor);
}
    final.a = base.a * gl_FrontMaterial.diffuse.a * gl_FrontMaterial.specular.a;

    gl_FragColor = final;
}